unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, bass, uSpectrum;

{*******************************************************************************}

type
  TChannelType = (ctUnknown, ctStream, ctMusic);

{*******************************************************************************}

const
  BassCount = 3;
  NumEQBands = 10;
  EQCount = NumEQBands - 1;
  EQCenter: array[0..EQCount] of 80..16000 = (80, 170, 310, 600, 1000, 3000, 6000, 10000, 12000, 14000);
  EQGainBass: array[0..BassCount-1] of -15..15 = (6, 5, 4);

type
  TEQArray = array[0..EQCount] of -15..15;

const
  EQGainDefault: TEQArray = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
  EQGainRock: TEQArray = (-2, 1, 3, 7, -2, -2, 0, 0, 7, 7);
  EQGainRap: TEQArray = (-2, 0, 1, 7, -2, -2, 0, 0, 7, 9);
  EQGainGrunge: TEQArray = (-5, 0, 0, -2, 0, 0, 2, 3, 0, -3);
  EQGainMetal: TEQArray = (-5, 0, 0, 0, 0, 0, 3, 0, 3, 2);
  EQGainTechno: TEQArray = (-7, 2, 4, -2, -2, -3, 0, 0, 5, 5);
  EQGainPop: TEQArray = (-1, 4, 5, 2, -1, -2, 0, 0, 4, 4);
  EQGainClassic: TEQArray = (0, 0, 0, 4, 5, 3, 7, 3, 0, 0);
  EQGainVoice: TEQArray = (-8, -6, -4, 1, 0, 0, 0, -4, -6, -8);

type
  TSNDEffects = set of (seFlange, seChorus, seEcho, seReverb, seBass, seEqualizer);

var
  RotPos: Single;

  FlangeHandle, ChorusHandle,
  EchoHandle, ReverbHandle: HFX;
  EQHandle: array [0..NumEQBands - 1] of HFX;

  FlangeParam: BASS_FXFLANGER;
  ChorusParam: BASS_FXCHORUS;
  EchoParam: BASS_FXECHO;
  ReverbParam: BASS_FXREVERB;
  EQParam: BASS_FXPARAMEQ;

  SNDEffects: TSNDEffects;

  EQGainCurrent: array[0..EQCount] of -15..15;

{*******************************************************************************}

type
  TfrmMain = class(TForm)
    dlgOpen: TOpenDialog;
    btnPlay: TButton;
    btnPause: TButton;
    btnStop: TButton;
    btnOpen: TButton;
    lbVolume_face: TLabel;
    barVolume: TTrackBar;
    lbPosLen: TLabel;
    barSeek: TTrackBar;
    lbFileName_face: TLabel;
    boxBackground: TBevel;
    tmrMain: TTimer;
    tmrFFT: TTimer;
    lbStatus_face: TLabel;
    boxSndEffects: TGroupBox;
    chFlange: TCheckBox;
    chReverb: TCheckBox;
    chChorus: TCheckBox;
    chEcho: TCheckBox;
    chBass: TCheckBox;
    chEqualizer: TCheckBox;
    eqPanel: TPanel;
    lb80_face: TLabel;
    lb170_face: TLabel;
    lb310_face: TLabel;
    lb600_face: TLabel;
    lb1k_face: TLabel;
    lb3k_face: TLabel;
    lb6k_face: TLabel;
    lb10k_face: TLabel;
    lb12k_face: TLabel;
    lb14k_face: TLabel;
    lb15db_face: TLabel;
    lb_15db_face: TLabel;
    lb0db_face: TLabel;
    barEQ80: TTrackBar;
    barEQ170: TTrackBar;
    barEQ310: TTrackBar;
    barEQ600: TTrackBar;
    barEQ1K: TTrackBar;
    barEQ3K: TTrackBar;
    barEQ6K: TTrackBar;
    barEQ10K: TTrackBar;
    barEQ12K: TTrackBar;
    barEQ14K: TTrackBar;
    boxEQSelect: TComboBox;
    btnReset: TButton;
    tmrTitle: TTimer;
    pnlTitle: TPanel;
    lbTags_face: TLabel;
    tmrWait: TTimer;
    boxPaint: TPaintBox;
    procedure FormCreate(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure barVolumeChange(Sender: TObject);
    procedure barSeekChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure tmrMainTimer(Sender: TObject);
    procedure tmrFFTTimer(Sender: TObject);
    procedure chFlangeClick(Sender: TObject);
    procedure chChorusClick(Sender: TObject);
    procedure chEchoClick(Sender: TObject);
    procedure chReverbClick(Sender: TObject);
    procedure chBassClick(Sender: TObject);
    procedure chEqualizerClick(Sender: TObject);
    procedure boxEQSelectChange(Sender: TObject);
    procedure barEQ80Change(Sender: TObject);
    procedure barEQ170Change(Sender: TObject);
    procedure barEQ310Change(Sender: TObject);
    procedure barEQ600Change(Sender: TObject);
    procedure barEQ1KChange(Sender: TObject);
    procedure barEQ3KChange(Sender: TObject);
    procedure barEQ6KChange(Sender: TObject);
    procedure barEQ10KChange(Sender: TObject);
    procedure barEQ12KChange(Sender: TObject);
    procedure barEQ14KChange(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure tmrTitleTimer(Sender: TObject);
    procedure tmrWaitTimer(Sender: TObject);
  private
    {**************************************************************************}
    Channel: DWORD;
    ChannelType: TChannelType;
    {**************************************************************************}
    FFileName: string;
    FFileLength: string;
    FTitleStr: string;
    procedure Error(const ErrMsg: string);
    procedure ExceptHook(Sender: TObject; E: Exception);
    {**************************************************************************}
    function BASS_PlaySoundFile(const FileName: string): Boolean;
    function BASS_GetMode: DWORD;
    function BASS_GetLength: DWORD;
    function BASS_GetPosition: DWORD;
    procedure BASS_SetPosition(const Value: DWORD);
    {**************************************************************************}
    procedure ClearSoundEffectHandles;
    procedure BASS_RestoreSoundEffect;
    procedure BASS_SetDefaultSoundEffect(const SNDEffects: TSNDEffects);
    procedure BASS_SetSoundEffects(const Value: TSNDEffects);
    procedure SetEQValues(EQArray: TEQArray);
    {**************************************************************************}
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

{$R RESOURCE.RES}

{*******************************************************************************}

uses
  bass_aac, bass_ac3, bass_alac, bass_ape, bass_mpc, bass_ofr, bass_spx,
  bass_tta, bass_wv, bassflac, basswma, tags, XpTheme;

const
  APPLICATION_NAME = 'BassPlayer (BASS library v2.2)';

{*******************************************************************************}

  ERR_FILE_NOT_FOUND           = 'File not found %s';
  ERR_LOAD                     = 'Loading error';
  ERR_BASS_DLL_NOT_LOADED      = 'BASS library not loaded';
  ERR_NOT_BASS_22              = 'BASS library version 2.2 not loaded';
  ERR_BASS_NOT_INIT            = 'BASS not initialized';
  ERR_DIRECTX_80_NOT_INSTALLED = 'DirectX 8.0 is not installed on your computer';
  ERR_OPEN_CH                  = 'Opening channel error';
  ERR_NOT_SUPPORTED_FORMAT     = 'Unsupported file format';

  ERR_FILE_NOT_FOUND_CODE      = 1;
  ERR_LOAD_CODE                = 2;
  ERR_BASS_DLL_NOT_LOADED_CODE = 3;
  ERR_NOT_BASS_22_CODE         = 4;
  ERR_BASS_NOT_INIT_CODE       = 5;

{*******************************************************************************}

var
  DIRECTX_80_NOT_INSTALLED: Boolean = False;
  EAX_SUPPORTED: Boolean = False;

  CHANGE_POS: Boolean = True;

  CHANGE_PRESET: Boolean = False;

{*******************************************************************************}

var
  LogoBMP: TBitmap;
  TempBMP: TBitmap;

procedure TfrmMain.Error(const ErrMsg: string);
begin
  MessageBox(Handle, PChar(ErrMsg), APPLICATION_NAME, MB_ICONWARNING or MB_SYSTEMMODAL);
end;

procedure TfrmMain.ExceptHook(Sender: TObject; E: Exception);
begin
  Error(E.Message);
end;

function MakeWndTrans(Wnd: HWND; nAlpha: Integer = 10): Boolean; 
type 
  TSetLayeredWindowAttributes = function(hwnd: HWND; crKey: COLORREF; bAlpha: Byte; 
    dwFlags: Longint): Longint; stdcall; 
const
  LWA_COLORKEY = 1;
  LWA_ALPHA = 2;
  WS_EX_LAYERED = $80000; 
var 
  hUser32: HMODULE; 
  SetLayeredWindowAttributes: TSetLayeredWindowAttributes; 
begin
  Result:= False;
  hUser32:= GetModuleHandle('USER32.DLL');
  if hUser32 <> 0 then 
  begin 
    @SetLayeredWindowAttributes := GetProcAddress(hUser32,'SetLayeredWindowAttributes'); 
    if @SetLayeredWindowAttributes <> nil then
    begin 
      SetWindowLong(Wnd, GWL_EXSTYLE, GetWindowLong(Wnd, GWL_EXSTYLE) or WS_EX_LAYERED);
      SetLayeredWindowAttributes(Wnd, 0, Trunc((255 / 100) * (100 - nAlpha)), LWA_ALPHA);
      Result := True; 
    end; 
  end; 
end;

function TfrmMain.BASS_PlaySoundFile(const FileName: string): Boolean; 
var
  ChannelInfo: BASS_CHANNELINFO;
begin
  Result:= False;
  
  case ChannelType of
    ctMusic: BASS_MusicFree(Channel);
    ctStream: BASS_StreamFree(Channel);
  end;
  
  Channel:= BASS_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
  if (Channel <> 0) then
    ChannelType:= ctStream
  else begin
    Channel:= BASS_WMA_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
    if (Channel <> 0) then
      ChannelType:= ctStream
    else begin
      Channel:= BASS_FLAC_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
      if (Channel <> 0) then
        ChannelType:= ctStream
      else begin
        Channel:= BASS_WV_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
        if (Channel <> 0) then
          ChannelType:= ctStream
        else begin
          Channel:= BASS_OFR_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
          if (Channel <> 0) then
            ChannelType:= ctStream
          else begin
            Channel:= BASS_AAC_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
            if (Channel <> 0) then
              ChannelType:= ctStream
            else begin
              Channel:= BASS_MP4_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
              if (Channel <> 0) then
                ChannelType:= ctStream
              else begin
                Channel:= BASS_AC3_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                if (Channel <> 0) then
                  ChannelType:= ctStream
                else begin
                  Channel:= BASS_ALAC_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                  if (Channel <> 0) then
                    ChannelType:= ctStream
                  else begin
                    Channel:= BASS_APE_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                    if (Channel <> 0) then
                      ChannelType:= ctStream
                    else begin
                      Channel:= BASS_MPC_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                      if (Channel <> 0) then
                        ChannelType:= ctStream
                      else begin
                        Channel:= BASS_SPX_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                        if (Channel <> 0) then
                          ChannelType:= ctStream
                        else begin
                          Channel:= BASS_TTA_StreamCreateFile(False, PChar(FileName), 0, 0, 0);
                          if (Channel <> 0) then
                            ChannelType:= ctStream
                          else begin
                            Channel:= BASS_MusicLoad(False, PChar(FileName), 0, 0, BASS_MUSIC_CALCLEN, 0);
                            if (Channel <> 0) then ChannelType:= ctMusic;
                          end;
                        end;
                      end;
                    end;
                  end;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  end;
  
  if Channel = 0 then Exit;
  BASS_ChannelGetInfo(Channel, ChannelInfo);
  if (ChannelInfo.flags and BASS_SAMPLE_8BITS) > 0 then
  begin
    BASS_MusicFree(Channel);
    BASS_StreamFree(Channel);
    Error(ERR_OPEN_CH);
    Exit;
  end;

  if (Channel <> 0) then
  begin
    ClearSoundEffectHandles;
    BASS_ChannelPlay(Channel, False);
    BASS_SetSoundEffects(SNDEffects);
  end;

  FFileLength:= FormatDateTime('nn:ss', BASS_GetLength / (1000 * 24 * 60 * 60));

  Result:= Channel <> 0;
end;

function TfrmMain.BASS_GetMode: DWORD;
begin
  Result:= BASS_ChannelIsActive(Channel);
end;

function TfrmMain.BASS_GetLength: DWORD;
var
  SongLength: Int64;
  FloatLen: FLOAT;
begin
   Result:= 0;
   SongLength:= 0;
   case ChannelType of
     ctUnknown: Exit;
     ctStream, ctMusic: SongLength:= BASS_ChannelGetLength(Channel);
   end;
   if SongLength < 0 then SongLength:= 0;
   FloatLen:= BASS_ChannelBytes2Seconds(Channel, SongLength);
   Result:= Round(1000 * FloatLen);
end;

function TfrmMain.BASS_GetPosition: DWORD;
var
  SongPos: Int64;
  FloatPos: FLOAT;
begin
  Result:= 0;
  case ChannelType of
    ctUnknown, ctMusic: Exit;
    ctStream:
    begin
      SongPos:= BASS_ChannelGetPosition(Channel);
      if SongPos < 0 then SongPos:= 0;
      FloatPos:= BASS_ChannelBytes2Seconds(Channel, SongPos);
      Result:= Round(1000 * FloatPos);
    end;
  end;
end;

procedure TfrmMain.BASS_SetPosition(const Value: DWORD);
var
  SongPos: Int64;
begin
  if ChannelType = ctUnknown then Exit;
  if Value > BASS_GetLength then Exit;
  if (BASS_GetMode = BASS_ACTIVE_STOPPED) then Exit;
  SongPos:= BASS_ChannelSeconds2Bytes(Channel, Value / 1000);
  case ChannelType of
    ctUnknown, ctMusic: Exit;
    ctStream: BASS_ChannelSetPosition(Channel, SongPos);
  end;
end;

{*******************************************************************************}


procedure TfrmMain.ClearSoundEffectHandles;
var
  I: Integer;
begin
  FlangeHandle:= 0;
  ChorusHandle:= 0;
  EchoHandle:= 0;
  ReverbHandle:= 0;
  for I:= 0 to (NumEQBands-1) do EQHandle[I]:= 0;
end;

procedure TfrmMain.BASS_RestoreSoundEffect;
var
  I: Integer;
begin
  if FlangeHandle <> 0 then
    if BASS_ChannelRemoveFX(Channel, FlangeHandle) then
      FlangeHandle:= 0;
  if ChorusHandle <> 0 then
    if BASS_ChannelRemoveFX(Channel, ChorusHandle) then
      ChorusHandle:= 0;
  if EchoHandle <> 0 then
    if BASS_ChannelRemoveFX(Channel, EchoHandle) then
      EchoHandle:= 0;
  if ReverbHandle <> 0 then
    if BASS_ChannelRemoveFX(Channel, ReverbHandle) then
      ReverbHandle:= 0;
  for I:= 0 to (NumEQBands-1) do
  begin
    if EQHandle[I] <> 0 then
      if BASS_ChannelRemoveFX(Channel, EQHandle[I]) then
        EQHandle[I]:= 0;
  end;
end;

procedure TfrmMain.BASS_SetDefaultSoundEffect(const SNDEffects: TSNDEffects);
begin

  if seFlange in SNDEffects then
  begin
    with FlangeParam do
    begin
      fWetDryMix:= 0;
      fDepth:= 25;
      fFeedback:= 0;
      fFrequency:= 0;
      lWaveform:= 1;
      fDelay:= 0;
      lPhase:= 0;
    end;
  end;

  if seChorus in SNDEffects then
  begin
    with ChorusParam do        
    begin
      fWetDryMix:= 0;
      fDepth:= 25;
      fFeedback:= 0;
      fFrequency:= 0;
      lWaveform:= 1;
      fDelay:= 0;
      lPhase:= 0;
    end;
  end;

  if seEcho in SNDEffects then
  begin
    with EchoParam do
    begin
      fWetDryMix:= 0;
      fFeedback:= 0;
      fLeftDelay:= 333;
      fRightDelay:= 333;
      lPanDelay:= False;
    end;
  end;

  if seReverb in SNDEffects then
  begin
    with ReverbParam do
    begin
      fInGain:= 0;
      fReverbMix:= 0;
      fReverbTime:= 1000;
      fHighFreqRTRatio:= 0.001;
    end;
  end;
  
  if seEqualizer in SNDEffects then
  begin
    EQParam.fBandwidth:= 9;
  end;
end;

procedure TfrmMain.BASS_SetSoundEffects(const Value: TSNDEffects);
var
  I: 0..NumEQBands - 1;
begin
  if (BASS_GetMode = BASS_ACTIVE_STOPPED) then Exit;
  BASS_RestoreSoundEffect;
  //------------------------
  if seFlange in Value then
  begin
    if FlangeHandle = 0 then
      FlangeHandle:= BASS_ChannelSetFX(Channel, BASS_FX_FLANGER, 1);
    if FlangeHandle <> 0 then
    begin
      if BASS_FXGetParameters(FlangeHandle, @FlangeParam) then
        BASS_FXSetParameters(FlangeHandle, @FlangeParam)
    end;
  end;
  //------------------------
  if seChorus in Value then
  begin
    if ChorusHandle = 0 then
      ChorusHandle:= BASS_ChannelSetFX(Channel, BASS_FX_CHORUS, 1);
    if ChorusHandle <> 0 then
    begin
      if BASS_FXGetParameters(ChorusHandle, @ChorusParam) then
        BASS_FXSetParameters(ChorusHandle, @ChorusParam)
    end;
  end;
  //------------------------
  if seEcho in Value then
  begin
    if EchoHandle = 0 then
      EchoHandle:= BASS_ChannelSetFX(Channel, BASS_FX_ECHO, 1);
    if EchoHandle <> 0 then
    begin
      if BASS_FXGetParameters(EchoHandle, @EchoParam) then
        BASS_FXSetParameters(EchoHandle, @EchoParam)
    end;
  end;
  //------------------------
  if seReverb in Value then
  begin
    if ReverbHandle = 0 then
      ReverbHandle:= BASS_ChannelSetFX(Channel, BASS_FX_REVERB, 1);
    if ReverbHandle <> 0 then
    begin
      if BASS_FXGetParameters(ReverbHandle, @ReverbParam) then
        BASS_FXSetParameters(ReverbHandle, @ReverbParam)
    end;
  end;
  //------------------------
  if seEqualizer in Value then
  begin
    for I:= 0 to EQCount do
    begin
      if EQHandle[I] = 0 then
        EQHandle[I]:= BASS_ChannelSetFX(Channel, BASS_FX_PARAMEQ, 1);
      EQParam.fCenter:= EQCenter[I];
      if (seBass in Value) and (I <= BassCount-1) then
        EQParam.fGain:= EQGainBass[I]
      else
        EQParam.fGain:= EQGainCurrent[I];  
      BASS_FXSetParameters(EQHandle[I], @EQParam);
    end;
  end else begin
    if seBass in Value then
    begin
      for I:= 0 to BassCount-1 do
      begin
        if EQHandle[I] = 0 then
          EQHandle[I]:= BASS_ChannelSetFX(Channel, BASS_FX_PARAMEQ, 1);
        EQParam.fCenter:= EQCenter[I];
        EQParam.fGain:= EQGainBass[I];
        BASS_FXSetParameters(EQHandle[I], @EQParam);
      end;
    end;
  end;
end;

procedure TfrmMain.SetEQValues(EQArray: TEQArray);
begin
  barEQ80.Position:= -EQArray[0];
  barEQ170.Position:= -EQArray[1];
  barEQ310.Position:= -EQArray[2];
  barEQ600.Position:= -EQArray[3];
  barEQ1K.Position:= -EQArray[4];
  barEQ3K.Position:= -EQArray[5];
  barEQ6K.Position:= -EQArray[6];
  barEQ10K.Position:= -EQArray[7];
  barEQ12K.Position:= -EQArray[8];
  barEQ14K.Position:= -EQArray[9];
end;

{*******************************************************************************}

procedure TfrmMain.FormCreate(Sender: TObject);
var
  BassInfo: BASS_INFO;
begin
  Application.OnException:= ExceptHook;
  Application.Title:= APPLICATION_NAME;
  Caption:= APPLICATION_NAME;

  if BASS_GetVersion <> DWORD(MAKELONG(2,2)) then
  begin
    Error(ERR_NOT_BASS_22);
    FatalExit(ERR_NOT_BASS_22_CODE);
  end;

  if not BASS_Init(1, 44100, BASS_DEVICE_3D, Handle, nil) then
  begin
    Error(ERR_BASS_NOT_INIT);
    FatalExit(ERR_BASS_NOT_INIT_CODE);
  end;
  
  BASS_Start;

  BASS_GetInfo(BassInfo);
  if BassInfo.dsver < 8 then
  begin
    Error(ERR_DIRECTX_80_NOT_INSTALLED);
    DIRECTX_80_NOT_INSTALLED:= True;

    chFlange.Enabled:= False;
    chChorus.Enabled:= False;
    chEcho.Enabled:= False;
    chReverb.Enabled:= False;
    chBass.Enabled:= False;
    chEqualizer.Enabled:= False;
  end;

  EAX_SUPPORTED:= BassInfo.eax;
  if EAX_SUPPORTED then
    BASS_SetEAXParameters(EAX_ENVIRONMENT_PSYCHOTIC, 1, 20, 2);

  dlgOpen.Filter:=
    'All supported formats|*.mp3;*.mp2;*.mp1;*.wav;*.ogg;*.mo3;*.it;*.xm;*.s3m;' +
    '*.mtm;*.mod;*.umx;*.wma;*.wm;*.ofr;*.ofs;*.flac;*.wv;*.aac;*.mp4;*.ac3;*.alac;*.ape;*.mpc;*.spx;*.tta';
  dlgOpen.Filter:= dlgOpen.Filter + '|' + 
    'MPEG Audio Files (*.mp3;*.mp2;*.mp1)|*.mp3;*.mp2;*.mp1|'+
    'Ogg Vorbis Audio (*.ogg)|*.ogg|'+
    'Microsoft Wave Audio (*.wav)|*.wav|'+
    'MOD Music (*.mo3;*.it;*.xm;*.s3m;*.mtm;*.mod;*.umx)|*.mo3;*.it;*.xm;*.s3m;*.mtm;*.mod;*.umx'+
    '|Windows Media Audio (*.wma;*.wm)|*.wma;*.wm'+
    '|OptimFROG Audio Files (*.ofr;*.ofs)|*.ofr;*.ofs'+
    '|Free Lossless Audio Files (*.flac)|*.flac'+
    '|WavPack Audio Files (*.wv)|*.wv'+
    '|FAAD Audio Files (.aac;*.mp4)|*.aac;*.mp4'+
    '|AC3 Audio Files (*.ac3)|*.ac3'+
    '|Apple Lossless Audio Files (*.alac)|*.alac'+
    '|Monkeys Audio Audio Files (*.ape)|*.ape'+
    '|Musepack Audio Files (*.mpc)|*.mpc'+
    '|Speex Audio Files (*.spx)|*.spx'+
    '|True Audio Files (*.tta)|*.tta';

  barVolume.Position:= BASS_GetVolume;

  LogoBMP:= TBitmap.Create;
  LogoBMP.Handle:= LoadBitmap(hInstance, 'LOGOBMP');
  TempBMP:= TBitmap.Create;
  TempBMP.Width:= boxPaint.Width;
  TempBMP.Height:= boxPaint.Height;
  Spectrum:= TSpectrum.Create(boxPaint.Width, boxPaint.Height);
  Spectrum.SetBackground(True, LogoBMP);
  Spectrum.Height:= boxPaint.Height - 5;
  Spectrum.Mask:= clGray;

  FFileLength:= '00:00';
  FTitleStr:= Caption;
  lbTags_face.Caption:= #32#32 + FTitleStr + ' ***';
  Application.Title:= FTitleStr + ' ***';

  boxEQSelect.ItemIndex:= 0;

  BASS_SetDefaultSoundEffect([seFlange, seChorus, seEcho, seReverb, seEqualizer]);

  //MakeWndTrans(Handle, 5);    
end;

procedure TfrmMain.btnPlayClick(Sender: TObject);
begin
  if (FFileName <> '') and (BASS_GetMode <> BASS_ACTIVE_PLAYING) then
    if (BASS_GetMode <> BASS_ACTIVE_PAUSED) then
    begin
      if not BASS_PlaySoundFile(FFileName) then
        Error(ERR_NOT_SUPPORTED_FORMAT);
    end else
      BASS_ChannelPlay(Channel, False);
end;

procedure TfrmMain.btnPauseClick(Sender: TObject);
begin
  if (BASS_GetMode <> BASS_ACTIVE_PAUSED) then
    BASS_ChannelPause(Channel);
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  if (BASS_GetMode <> BASS_ACTIVE_STOPPED) then
    BASS_ChannelStop(Channel);
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
  if dlgOpen.Execute then
  begin
    FFileName:= dlgOpen.FileName;
    if not BASS_PlaySoundFile(FFileName) then
      Error(ERR_NOT_SUPPORTED_FORMAT)
    else begin
      lbFileName_face.Caption:= ExtractFileName(FFileName);
      FTitleStr:= TAGS_Read(Channel,
        '%IFV2(%ARTI,%ICAP(%ARTI),Unknown artist) - %IFV2(%TITL,%ICAP(%TITL),Untitled)');
      if FTitleStr = '' then FTitleStr:= 'Unknown artist - Untitled';
    end;
  end;
end;

procedure TfrmMain.barVolumeChange(Sender: TObject);
begin
  BASS_SetVolume(barVolume.Position);
end;

procedure TfrmMain.barSeekChange(Sender: TObject);
var
  PD: Single;
begin
  if CHANGE_POS then
  begin
    tmrMain.Enabled:= False;
    PD:= BASS_GetLength / barSeek.Max;
    BASS_SetPosition(Round(barSeek.Position * PD));
    tmrMain.Enabled:= True;
  end;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  tmrMain.Enabled:= True;
  btnOpen.SetFocus;
end;

procedure TfrmMain.tmrMainTimer(Sender: TObject);
var
  ActiveMode: DWORD;  
begin
  ActiveMode:= BASS_GetMode;
  
  case ActiveMode of
    BASS_ACTIVE_STOPPED,
    BASS_ACTIVE_STALLED:
    begin
      if FileExists(FFileName) then
        btnPlay.Enabled:= True
      else
        btnPlay.Enabled:= False;
      btnPause.Enabled:= False;
      btnStop.Enabled:= False;
      lbStatus_face.Caption:= 'Stopped';
    end;
    BASS_ACTIVE_PLAYING:
    begin
      btnPlay.Enabled:= False;
      btnPause.Enabled:= True;
      btnStop.Enabled:= True;
      lbStatus_face.Caption:= 'Playing';
    end;
    BASS_ACTIVE_PAUSED:
    begin
      btnPlay.Enabled:= True;
      btnPause.Enabled:= False;
      btnStop.Enabled:= True;
      lbStatus_face.Caption:= 'Paused';
    end;
  end;
  
  if (ActiveMode <> BASS_ACTIVE_STOPPED) then
  begin
    if not barSeek.Enabled then
      barSeek.Enabled:= True;
    CHANGE_POS:= False;
    barSeek.Position:= (BASS_GetPosition * DWORD(barSeek.Max)) div BASS_GetLength;
    CHANGE_POS:= True;
    lbPosLen.Caption:=
      FormatDateTime('nn:ss', BASS_GetPosition / (1000 * 24 * 60 * 60)) + ' -> '+ FFileLength;
    if (BASS_GetMode = BASS_ACTIVE_PAUSED) then
    begin
      BitBlt(boxPaint.Canvas.Handle, 0, 0, TempBMP.Width,
        TempBMP.Height, TempBMP.Canvas.Handle, 0, 0, SRCCOPY);
      tmrFFT.Enabled:= False;
    end else begin
      BitBlt(TempBMP.Canvas.Handle, 0, 0, boxPaint.Width,
        boxPaint.Height, boxPaint.Canvas.Handle, 0, 0, SRCCOPY);
      tmrFFT.Enabled:= True;
    end;
  end else begin
    barSeek.Position:= 0;
    barSeek.Enabled:= False;
    tmrFFT.Enabled:= False;
    lbPosLen.Caption:= '00:00 -> '+ FFileLength;
    BitBlt(boxPaint.Canvas.Handle, 0, 0, LogoBMP.Width,
      LogoBMP.Height, LogoBMP.Canvas.Handle, 0, 0, SRCCOPY);
  end;
end;

procedure TfrmMain.tmrFFTTimer(Sender: TObject);
var
  FFTFata: TFFTData;
begin
  if BASS_ChannelIsActive(Channel) <> BASS_ACTIVE_PLAYING then Exit;
  BASS_ChannelGetData(Channel, @FFTFata, BASS_DATA_FFT1024);
  Spectrum.Draw(boxPaint.Canvas.Handle, FFTFata, 2, -2);
end;

{*******************************************************************************}

procedure TfrmMain.chFlangeClick(Sender: TObject);
begin
  if chFlange.Checked then
    Include(SNDEffects, seFlange)
  else
    Exclude(SNDEffects, seFlange);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.chChorusClick(Sender: TObject);
begin
  if chChorus.Checked then
    Include(SNDEffects, seChorus)
  else
    Exclude(SNDEffects, seChorus);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.chEchoClick(Sender: TObject);
begin
  if chEcho.Checked then
    Include(SNDEffects, seEcho)
  else
    Exclude(SNDEffects, seEcho);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.chReverbClick(Sender: TObject);
begin
  if chReverb.Checked then
    Include(SNDEffects, seReverb)
  else
    Exclude(SNDEffects, seReverb);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.chBassClick(Sender: TObject);
begin
  if chBass.Checked then
    Include(SNDEffects, seBass)
  else
    Exclude(SNDEffects, seBass);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.chEqualizerClick(Sender: TObject);
begin
  if chEqualizer.Checked then
  begin
    Include(SNDEffects, seEqualizer);
    frmMain.ClientHeight:= 419;
    eqPanel.Visible:= True;
    frmMain.Position:= poScreenCenter;
  end else begin
    Exclude(SNDEffects, seEqualizer);
    frmMain.ClientHeight:= 242;
    eqPanel.Visible:= False;
    frmMain.Position:= poScreenCenter;
  end;
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.boxEQSelectChange(Sender: TObject);
var
  I: Integer;
begin
  CHANGE_PRESET:= True;
  case boxEQSelect.ItemIndex of
    0, 9:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainDefault[I];
      SetEQValues(EQGainDefault);
    end;
    1:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainRock[I];
      SetEQValues(EQGainRock);
    end;
    2:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainRap[I];
      SetEQValues(EQGainRap);
    end;
    3:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainGrunge[I];
      SetEQValues(EQGainGrunge);
    end;
    4:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainMetal[I];
      SetEQValues(EQGainMetal);
    end;
    5:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainTechno[I];
      SetEQValues(EQGainTechno);
    end;
    6:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainPop[I];
      SetEQValues(EQGainPop);
    end;
    7:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainClassic[I];
      SetEQValues(EQGainClassic);
    end;
    8:
    begin
      for I:= 0 to EQCount do
        EQGainCurrent[I]:= EQGainVoice[I];
      SetEQValues(EQGainVoice);
    end;
  end;
  CHANGE_PRESET:= False;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ80Change(Sender: TObject);
begin
  EQGainCurrent[0]:= -barEQ80.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ170Change(Sender: TObject);
begin
  EQGainCurrent[1]:= -barEQ170.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ310Change(Sender: TObject);
begin
  EQGainCurrent[2]:= -barEQ310.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ600Change(Sender: TObject);
begin
  EQGainCurrent[3]:= -barEQ600.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ1KChange(Sender: TObject);
begin
  EQGainCurrent[4]:= -barEQ1K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ3KChange(Sender: TObject);
begin
  EQGainCurrent[5]:= -barEQ3K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ6KChange(Sender: TObject);
begin
  EQGainCurrent[6]:= -barEQ6K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ10KChange(Sender: TObject);
begin
  EQGainCurrent[7]:= -barEQ10K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ12KChange(Sender: TObject);
begin
  EQGainCurrent[8]:= -barEQ12K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.barEQ14KChange(Sender: TObject);
begin
  EQGainCurrent[9]:= -barEQ14K.Position;
  if not CHANGE_PRESET then
    boxEQSelect.ItemIndex:= 9;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
var
  I: Integer;
begin
  for I:= 0 to EQCount do
    EQGainCurrent[I]:= EQGainDefault[I];
  SetEQValues(EQGainDefault);
  boxEQSelect.ItemIndex:= 0;
  Include(SNDEffects, seEqualizer);
  BASS_SetSoundEffects(SNDEffects);
end;

{******************************************************************************}

var
  I1: Integer = 0;
  I2: Integer = 0;
  L1: Integer = 0;
  L2: Integer = 0;
  L3: Integer = 0;
  NFS: Integer = 0;
  ZF: Integer = 0;
  W: Integer;

procedure TfrmMain.tmrTitleTimer(Sender: TObject);
var
  szTitle, szTmp: string;
  J, N: Integer;
label
  C2, C3;
begin
  szTmp:= Format('%s ***', [FTitleStr]);
  L1:= Length(szTmp);
  if (I1 > L1) then
  begin
    if (L2 = L1) then
    begin
      L2:= 0;
      if (I2 = 1) then
      begin
        I1:= 0;
        ZF:= 1;
        goto C2;
      end;
    end;
    I2:= 1;
    Inc(L2);
    Inc(L3);
  end else begin
    C2:
    if (ZF = 0) then
    begin
      Inc(I1);
      I2:= I1;
      L2:= L1;
    end else begin
      I2:= 1;
      L2:= L1; 
      Inc(L3);
    end;
  end;
  szTitle:= Copy(szTmp, I2, L2);
  if (I2 = 1) and (NFS = 1) then
  begin
    N:= L1 - L3 + W;
    if (N < 1) then
    begin
      I1:= 0;
      I2:= 0;
      L2:= 0;
      L3:= 0;
      NFS:= 0;
      ZF:= 0;
      Exit;
    end;
    for J:= 1 to N do
      szTitle:= #32 + szTitle;
  end;
  NFS:= 1;
  Application.Title:= szTmp;
  lbTags_face.Caption:= #32#32 + szTitle;
  Application.ProcessMessages;
end;

procedure TfrmMain.tmrWaitTimer(Sender: TObject);
begin
  W:= Round(lbTags_face.Width / 6);
  tmrTitle.Enabled:= True;
  tmrWait.Enabled:= False;
end;

end.
