{
Tags Copyright (c) by Wraith, 2k5
Delphi Wrapper written by Chris Troesken 
}

unit tags;

interface

uses
  Windows;

{ Parameters:
    handle  - BASS handle, obtained normally via BASS_StreamCreatexx() call,
              or any other handle, on which BASS_StreamGetTags() can be called.
    fmt         - format string, see below.}
function TAGS_Read(handle: DWORD; const fmt: PChar): PChar; stdcall; external 'tags.dll';

function TAGS_GetLastErrorDesc: PChar; stdcall; external 'tags.dll';
function TAGS_GetVersion(): DWORD; stdcall; external 'tags.dll';

{
Format string:
==============
       1. format string can contain the following things:
       - plain text like "Some song". This text is merely copied to the output.
       - special identifier, beginning with '%' to substitute for the tag value:
         "%TITL"  - song title;
         "%ARTI"  - song artist;
         "%ALBM"  - album name;
         "%GNRE"  - song genre;
         "%YEAR"  - song/album year;
         "%CMNT"  - comment;
         "%TRCK"  - track number;
       - expression:
         "%IFV1(x,a)" - if x is not empty, then %IFV1() evaluates to a,
                        or to an empty string otherwise;
         "%IFV2(x,a,b)" - if x is not empty, then %IFV2() evaluates to a,
                        else to b;
         "%IUPC(x)" - brings x to uppercase, so "%IUPC(foO)" yields "FOO";
         "%ILWC(x)" - brings x to lowercase, so "%ILWC(fOO)" yields "foo";
         "%ICAP(x)" - capitalizes first letter in each word of x, so
                      "%ICAP(FoO bAR)" yields "Foo Bar";

       - escaped symbols:
         "%%" - "%"
         "%(" - "("
         "%," - ","
         "%)" - ")"

       Example. Assume we have the following information in the tag:
             Title: "Nemo"
             Artist: "nightwish"
             Album: "Once"
             Track: "3"
             Year: "2004"

         Format string: "%IFV1(%TRCK,%TRCK. )%IFV2(%ARTI,%ICAP(%ARTI),no artist) - %IFV2(%TITL,%ICAP(%TITL) -,no title -) %IFV1(%ALBM,%IUPC(%ALBM))%IFV1(%YEAR, %(%YEAR%))"
         Output: "3.- Nightwish - Nemo - ONCE (2004)"

         if 'Artist' and 'Title' are empty, the output will be:
         "3. - no artist - no title - ONCE (2004)"

         if only 'Track' is empty, the output will be
         "Nightwish - Nemo - ONCE (2004)"

       Caution:
         "%IFV2(sometext ,a,b)" always evaluates to a, because a space after
         "sometext" causes the condition string to be not empty. This is
         intentional.

       Another caution:
         "symbols '%(,)' are reserved, that is, they must be escaped if they are
         inteded to appear in the output. See the above example: the parens
         around %YEAR are escaped with '%' to prevent misinterpretation.

    2. Ill-formed string causes some error message to appear in the output, but
          don't count on it too much...

    3. Great warning:
      The output string is allocated by tags.dll, so deleting the pointer may
      crash the program. DON'T! In addition to this, the function is NOT THREAD SAFE.
      It is user's responsibility to serialize the execution (a critical section
      would suffice). The output string itself is valid till next TAGS_Read() call.

    4. If mp3 contains both ID3v1 and ID3v2 tags, ID3v2 data takes precedence.           
}

implementation
end.

